/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.jdbc.handlers.filters.Filter;
import org.wso2.registry.jdbc.Repository;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Registry;
import org.wso2.registry.users.UserRealm;

import javax.sql.DataSource;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class HandlerManager {

    private Map<Filter, Handler> handlerMap = new HashMap();

    private DataSource dataSource;
    private UserRealm userRealm;
    private Registry registry;
    private Repository repository;

    public HandlerManager(
            DataSource dataSource, UserRealm userRealm, Registry registry, Repository repository) {
        this.dataSource = dataSource;
        this.userRealm = userRealm;
        this.registry = registry;
        this.repository = repository;
    }

    public void addHanlder(Filter filter, Handler handler) {

        filter.setRepository(repository);
        
        handler.setDataSource(dataSource);
        handler.setRealm(userRealm);
        handler.setRegistry(registry);
        handler.setRepository(repository);
        
        handlerMap.put(filter, handler);
    }

    public Resource get(RequestContext requestContext) throws RegistryException {

        Resource resource = null;

        Iterator<Filter> filters = handlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleGet(requestContext)) {
                resource = handlerMap.get(filter).get(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }

        return resource;
    }

    public String put(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = handlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handlePut(requestContext)) {
                handlerMap.get(filter).put(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }

        if (requestContext.isProcessingComplete()) {
            return requestContext.getActualPath();
        } else {
            return null;
        }
    }

    public String importResource(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = handlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleImportResource(requestContext)) {
                handlerMap.get(filter).importResource(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }

        if (requestContext.isProcessingComplete()) {
            return requestContext.getActualPath();
        } else {
            return null;
        }
    }

    public void delete(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = handlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleDelete(requestContext)) {
                handlerMap.get(filter).delete(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }
    }

    public void putChild(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = handlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handlePutChild(requestContext)) {
                handlerMap.get(filter).putChild(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }
    }

    public void importChild(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = handlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleImportChild(requestContext)) {
                handlerMap.get(filter).importChild(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }
    }
}
